package com.engine.kq.biz;

import com.alibaba.fastjson.JSON;
import com.cloudstore.dev.api.bean.MessageBean;
import com.cloudstore.dev.api.bean.MessageType;
import com.cloudstore.dev.api.util.Util_Message;
import com.engine.kq.entity.KQGroupEntity;
import com.engine.kq.log.KQLog;
import com.engine.msgcenter.biz.ConfigManager;
import com.engine.msgcenter.biz.WeaMessageTypeConfig;
import com.google.common.collect.Sets;
import com.weaver.general.Util;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.hrm.common.Tools;
import weaver.hrm.resource.ResourceComInfo;
import weaver.interfaces.schedule.BaseCronJob;
import weaver.systeminfo.SystemEnv;

import java.util.*;

public class KQYesterdayAbnormalRemindJob extends BaseCronJob {

    private BaseBean log = new BaseBean();

    private KQLog kqLog = new KQLog();

    public void execute() {
        String currentDate = Tools.getCurrentDate();
        String yesterDay = DateUtil.addDate(currentDate, -1);
        kqLog.info(">>>>>>>>>>>>>>>>KQYesterdayAbnormalRemindJob>>>>>>>>>>>>>>begin>>>>>>>>>>>>>>>>");
        handleKqRemind(yesterDay);
        kqLog.info(">>>>>>>>>>>>>>>>KQYesterdayAbnormalRemindJob>>>>>>>>>>>>>>end>>>>>>>>>>>>>>>>");

        // 一次性需求 #3416012# 补充考勤异常提醒2024.12.01 - 2024.12.08 之间未开启开关丢失的提醒
//        List<String> supDates = Arrays.asList("2024-12-01","2024-12-02","2024-12-03","2024-12-04","2024-12-05","2024-12-06","2024-12-07","2024-12-08");
//        supDates.forEach(date -> handleKqRemind(date));
    }


    private void handleKqRemind(String date) {
        try {
            ResourceComInfo resourceComInfo = new ResourceComInfo();
            KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
            Map<String, StringBuilder> managerMessages = new HashMap<>();
            Map<String, StringBuilder> managerMessagesUUID = new HashMap<>();
            Map<String,  Map<String, Double>> yesterdayAbnormalData = new KqYesterdayAbnormalRemindBiz().getYesterdayAbnormalData(date, date);
            String curDateTime = DateUtil.getDateTime();
            RecordSet recordSet = new RecordSet();
            for(Map.Entry<String,  Map<String, Double>> entry : yesterdayAbnormalData.entrySet()) {
                String resourceId = entry.getKey();
                Map<String, Double> value = entry.getValue();

                KQGroupEntity kQGroupEntity = kqGroupMemberComInfo.getUserKQGroupInfo(resourceId, date);
                String abnormalremind = Util.null2String(kQGroupEntity.getAbnormalremind());
                if(!"1".equals(abnormalremind)){
                    kqLog.info(">>>>>>>>>>>>>>>>KQYesterdayAbnormalRemindJob>>>>>>>>>>>>>>abnormalremind is closed>>>>>>>>>>>>>>>>"+resourceId);
                    continue;
                }

                kqLog.info(">>>>>>>>>>>>>>>>KQYesterdayAbnormalRemindJob>>>>>>>>>>>>>>value>>>>>>>>>>>>>>>>"+value);
                double late = value.get("late") == null ? 0.0 : value.get("late");
                double leaveearly = value.get("leaveearly") == null ? 0.0 : value.get("leaveearly");
                double absent = value.get("absent") == null ? 0.0 : value.get("absent");
                double forgot = value.get("forgot") == null ? 0.0 : value.get("forgot");
                double gravelate = value.get("gravelate") == null ? 0.0 : value.get("gravelate");
                double graveleaveearly = value.get("graveleaveearly") == null ? 0.0 : value.get("graveleaveearly");
                String content = getLabelName(97) +"："+date+" " + getLabelName(1867)+"："+resourceComInfo.getLastname(resourceId) +" "+ getLabelName(24770)+"：";
                StringBuffer remindContentStr = new StringBuffer();
                StringBuffer sb = new StringBuffer();

                if(late > 0) {
                    sb.append(getLabelName(20081)) ;//迟到
                    remindContentStr.append(getLabelName(390055)).append("：").append(late).append(getLabelName(518516));
                }

                if(gravelate > 0){
                    if(sb.length() > 0) {
                        sb.append( ",");
                    }
                    sb.append(getLabelName(500546)) ;//严重迟到
                    if(remindContentStr.length() >0) {
                        remindContentStr.append(",");
                    }
                    remindContentStr.append(getLabelName(390056)).append("：").append(gravelate).append(getLabelName(518516));
                }

                if(leaveearly > 0){
                    if(sb.length() > 0) {
                        sb.append( ",");
                    }
                    sb.append(getLabelName(20082)) ;//早退
                    if(remindContentStr.length() >0) {
                        remindContentStr.append(",");
                    }
                    remindContentStr.append(getLabelName(390057)).append("：").append(leaveearly).append(getLabelName(518516));
                }

                if(graveleaveearly > 0){
                    if(sb.length() > 0) {
                        sb.append( ",");
                    }
                    sb.append(getLabelName(500547)) ;//严重早退
                    if(remindContentStr.length() >0) {
                        remindContentStr.append(",");
                    }
                    remindContentStr.append(getLabelName(390058)).append("：").append(graveleaveearly).append(getLabelName(518516));
                }

                if(absent > 0) {
                    if(sb.length() > 0) {
                        sb.append( ",");
                    }
                    sb.append( getLabelName(20085));//旷工

                    if(remindContentStr.length() >0) {
                        remindContentStr.append(",");
                    }
                    remindContentStr.append(getLabelName(390059)).append("：").append(absent).append(getLabelName(518516));
                }
                if(forgot > 0) {
                    if(sb.length() > 0) {
                        sb.append( ",");
                    }
                    sb.append(getLabelName(20086));//漏签

                    if(remindContentStr.length() >0) {
                        remindContentStr.append(",");
                    }
                    remindContentStr.append(getLabelName(20086)).append(getLabelName(21551)).append("：").append(forgot).append(getLabelName(518516));
                }
                if(sb.length() > 0) {
                    content += sb.toString();
                    content += "<br>"+remindContentStr.toString();
                } else {
                    continue;
                }
                kqLog.info(">>>>>>>>>>>>>>>>KQYesterdayAbnormalRemindJob>>>>>>>>>>>>>>content>>>>>>>>>>>>>>>>"+content);
                String uuid = UUID.randomUUID().toString().replace("-", "");

                appNotification(resourceId, content, uuid);
                insertMessageLog(resourceId, content, resourceId, curDateTime, uuid, recordSet);


                String managerID = resourceComInfo.getManagerID(resourceId);
                if (managerID != null && !"".equals(managerID) && Util.getIntValue(managerID, 0) > 0)  {
                    String managerUUID = "";
                    if(managerMessagesUUID.containsKey(managerID)) {
                        managerUUID = managerMessagesUUID.get(managerID).toString();
                    } else {
                        managerUUID = UUID.randomUUID().toString().replace("-", "");
                        managerMessagesUUID.put(managerID, new StringBuilder(managerUUID));
                    }
                    insertMessageLog(resourceId, content, managerID, curDateTime, managerUUID, recordSet);
                    managerMessages.putIfAbsent(managerID, new StringBuilder());
                    managerMessages.get(managerID).append(content).append("<br><br>");
                    
                }
            }
            // 统一发送给上级
            for (Map.Entry<String, StringBuilder> entry : managerMessages.entrySet()) {
                String managerID = entry.getKey();
                String content = entry.getValue().toString();
                String managerUUID = managerMessagesUUID.get(managerID).toString();
                appNotification(managerID, content, managerUUID);
            }
        } catch (Exception e) {
            log.writeLog(e);
        }
    }

    private static String getLabelName(int labelId) {
        return SystemEnv.getHtmlLabelName(labelId, weaver.general.ThreadVarLanguage.getLang());
    }

	private void insertMessageLog(String resourceId, String content, String receiverId, String sendTime, String uuid, RecordSet rs) {

    String sql = "INSERT INTO abnormalremindmsg_log (resourceId, content, sendTime, receiverId,uuid) VALUES (?, ?, ?, ?, ?)";
    try {
        rs.executeUpdate(sql, resourceId, content, sendTime, receiverId, uuid);
    } catch (Exception e) {
        log.writeLog(e);
        kqLog.info("KQYesterdayAbnormalRemindJob::insertMessageLog: error inserting log for resourceId=" + resourceId + ", receiverId=" + receiverId, e);
    }
}


    public void appNotification(String resourceId,String content, String uuid) {
        try {
            String detailTitle = "539214";
            String title = content;
            int createrId = 1;
            String pcUrl = "/spa/hrm/static4attendance/index.html#/attendance/abnormalremindMsg?uuid="+uuid;
            String mobileUrl = "/spa/hrm/static4mobile/index.html#/abnormalremindMsg?uuid="+uuid;
            ConfigManager configManager = new ConfigManager();
            // 2、默认规则检查用户配置,返回后台自定义消息类型和对应通过配置的用户
            //    @param singPath 与需求变更之前的path参数含义一致,代表该消息类型详细配置中唯一标志值,各模块参考自己的值(有疑问请联系云商店-田泽法)
            //    @param userid 按默认规则检查的用户配置信息(检查多人参考重载方法)
            Map<WeaMessageTypeConfig, List<String>> accessConfig = configManager.defaultRuleCheckConfig(MessageType.HRM_KQ_REMIND, resourceId, null);

            // 3、遍历自定义消息类型集合
            for (Map.Entry<WeaMessageTypeConfig, List<String>> entry : accessConfig.entrySet()) {
                // 4、构造消息实体
                MessageBean bean = Util_Message.createMessage(MessageType.HRM_KQ_REMIND, 0, title, detailTitle, content, pcUrl, mobileUrl, createrId);

                // 5、获取新的自定义消息类型
                WeaMessageTypeConfig config = entry.getKey();

                // 6、新的自定义消息类型相关信息通知到消息实体bean上
                bean.setMessageConfig(config);

                // 7、设置检查配置通过需要发送消息的用户
                bean.setUserList(Sets.newHashSet(entry.getValue()));

                // 8、发送消息
                Util_Message.sendAndpublishMessage(bean);
                kqLog.info("in>>>>KQYesterdayAbnormalRemindJob>>>>>>appNotification=bean::"+ JSON.toJSONString(bean));
            }
        } catch (Exception e) {
            log.writeLog(e);
        }
    }

}
