package com.engine.kq.biz;

import com.engine.kq.log.KQLog;
import weaver.conn.RecordSet;
import weaver.general.Util;

import java.util.HashMap;
import java.util.Map;

/**
 * 考勤异常提醒公共接口
 */
public class KqYesterdayAbnormalRemindBiz {

    private KQLog kqLog = new KQLog();
    /**
     * 获取昨天考勤异常相关数据
     * @return
     */
    public Map<String,  Map<String, Double>> getYesterdayAbnormalData(String fromDate, String toDate) {
        return getYesterdayAbnormalData(fromDate, toDate, null);
    }

    public Map<String,  Map<String, Double>> getYesterdayAbnormalData(String fromDate, String toDate, String resourceId) {
        kqLog.info("KqAbnormalRemindBiz::getYesterdayAbnormalData:fromDate"+fromDate+":toDate:"+toDate+":resourceId:"+resourceId);
        RecordSet rs = new RecordSet();
        Map<String, Map<String, Double>> yesterdayAbnormalMap = new HashMap<>();
        String forgotBeginWorkCheck_field = " sum(b.forgotBeginWorkCheck) ";

        if(rs.getDBType().equalsIgnoreCase("oracle") || rs.getDBType().equalsIgnoreCase("gs")) {
            forgotBeginWorkCheck_field = " sum(nvl(b.forgotbeginworkcheckmins,0))  ";
        }else if((rs.getDBType()).equalsIgnoreCase("mysql")){
            forgotBeginWorkCheck_field = " sum(ifnull(b.forgotbeginworkcheckmins,0)) ";
        }else {
            forgotBeginWorkCheck_field = " sum(isnull(b.forgotbeginworkcheckmins,0)) ";
        }
        String backFields = " a.id,a.lastname,a.workcode,a.dsporder,b.resourceid,a.subcompanyid1 as subcompanyid,a.departmentid,a.jobtitle," +
                " sum(b.beLateMins) as beLateMins, " +
                " sum(b.graveBeLateMins) as graveBeLateMins," +
                " sum(b.leaveEarlyMins) as leaveEarlyMins, " +
                " sum(b.graveLeaveEarlyMins) as graveLeaveEarlyMins, " +
                " sum(b.absenteeismMins) as absenteeismMins, sum(b.forgotcheckmins)+"+forgotBeginWorkCheck_field+" as forgotCheckMins ";

        if(rs.getDBType().equals("oracle")){
            backFields = 	"/*+ index(kq_format_total IDX_KQ_FORMAT_TOTAL_KQDATE) */ "+backFields;
        }
        String sqlFrom = " from hrmresource a, kq_format_detail b where a.id= b.resourceid and b.kqdate >='"+fromDate+"' and b.kqdate <='"+toDate+"'";
        if(resourceId != null) {
            sqlFrom += " and (a.accounttype = 0 or a.accounttype is null) and  a.id="+resourceId;
        }
        String groupBy = " group by a.id,a.lastname,a.workcode,a.dsporder,b.resourceid,a.subcompanyid1,a.departmentid,a.jobtitle ";
        String sql = backFields + sqlFrom + groupBy;
        sql = " select " + sql;
        kqLog.info("KqAbnormalRemindBiz::sql:"+sql);
        rs.executeQuery(sql);
        while (rs.next()) {
            Map<String, Double> abnormalDataMap = new HashMap<>();
            String userId = rs.getString("id");
            double beLateMins = Util.getDoubleValue(rs.getString("beLateMins"), 0.0);//迟到
            double graveBeLateMins = Util.getDoubleValue(rs.getString("graveBeLateMins"), 0.0);//严重迟到
            double leaveEarlyMins = Util.getDoubleValue(rs.getString("leaveEarlyMins"), 0.0);//早退
            double graveLeaveEarlyMins = Util.getDoubleValue(rs.getString("graveLeaveEarlyMins"), 0.0);//严重早退
            double absenteeismMins = Util.getDoubleValue(rs.getString("absenteeismMins"), 0.0);//旷工
            double forgotCheck = Util.getDoubleValue(rs.getString("forgotCheckMins"), 0.0);//漏签
            if(beLateMins> 0) {
                abnormalDataMap.put("late", beLateMins);
            }
            if(graveBeLateMins> 0) {
                abnormalDataMap.put("gravelate", graveBeLateMins);
            }
            if(leaveEarlyMins> 0) {
                abnormalDataMap.put("leaveearly", leaveEarlyMins);
            }
            if(graveLeaveEarlyMins> 0) {
                abnormalDataMap.put("graveleaveearly", graveLeaveEarlyMins);
            }
            if(absenteeismMins> 0) {
                abnormalDataMap.put("absent", absenteeismMins);
            }
            if(forgotCheck> 0) {
                abnormalDataMap.put("forgot", forgotCheck);
            }
            if(!abnormalDataMap.isEmpty()) {
                yesterdayAbnormalMap.put(userId, abnormalDataMap);
            }
        }
        kqLog.info("KqAbnormalRemindBiz::getYesterdayAbnormalData:yesterdayAbnormalMap"+yesterdayAbnormalMap);
        return yesterdayAbnormalMap;
    }
}
