package com.engine.kq.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.engine.kq.bean.ImportSetting;
import com.engine.kq.bean.KqLockAttenda;
import com.engine.kq.biz.*;
import com.engine.kq.cmd.shiftmanagement.toolkit.ShiftManagementToolKit;
import com.engine.kq.entity.KQScheduleSignEntity;
import com.engine.kq.entity.WorkTimeEntity;
import com.engine.kq.enums.KqSplitFlowTypeEnum;
import com.engine.kq.log.KQLog;
import com.engine.kq.timer.KQQueue;
import com.engine.kq.timer.KQTaskBean;
import com.engine.kq.wfset.bean.ProcessChangeSplitBean;
import com.engine.kq.wfset.bean.SplitBean;
import com.engine.kq.wfset.util.*;
import com.engine.workflow.biz.freeNode.FreeNodeBiz;
import com.engine.workflow.biz.publicApi.RequestOperateBiz;
import com.engine.workflow.biz.workflowOvertime.OvertimeBiz;
import com.engine.workflow.constant.PAResponseCode;
import com.engine.workflow.entity.core.RequestInfoEntity;
import com.engine.workflow.entity.publicApi.PAResponseEntity;
import com.engine.workflow.entity.publicApi.ReqOperateRequestEntity;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.wbi.util.StringUtil;
import weaver.conn.BatchRecordSet;
import weaver.conn.RecordSet;
import weaver.conn.RecordSetDataSource;
import weaver.file.Prop;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.hrm.common.Tools;
import weaver.hrm.resource.ResourceComInfo;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.RequestInfo;
import weaver.systeminfo.SystemEnv;
import weaver.wechat.util.DateUtil;
import weaver.workflow.agent.AgentManager;
import weaver.workflow.monitor.Monitor;
import weaver.workflow.monitor.MonitorDTO;
import weaver.workflow.workflow.WfForceOver;
import weaver.workflow.workflow.WfFunctionManageUtil;
import weaver.workflow.workflow.WorkflowComInfo;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;


@SuppressWarnings("unchecked")
public class KQLockAttendaUtil {

    private static final String LOCK_TABLE = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_TABLE"));
    private static final String LOCK_COM = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_COM"));
    private static final String LOCK_SUBCOM = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_SUBCOM"));
    private static final String LOCK_DEPT = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_DEPT"));
    private static final String LOCK_RESOURCE = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_RESOURCE"));
    private static final String LOCK_FROMDATE = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_FROMDATE"));
    private static final String LOCK_TODATE = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_TODATE"));
    private static final String LOCK_LOCK = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_LOCK"));

    private static final String LOCK_DEGREE = Util.null2String(Prop.getPropValue("kqLockAttenda", "LOCK_DEGREE"));
    private static KQLog kqLog = new KQLog();

    public List<KqLockAttenda> sdkqbb_map;

    public static final ExecutorService EXECUTOR_SERVICE = new ThreadPoolExecutor(1, 1, 0L, TimeUnit.MILLISECONDS, new ArrayBlockingQueue<>(100));

    public KQLockAttendaUtil() {
        this.sdkqbb_map = this.initModuleData();
    }


    public List<KqLockAttenda> initModuleData() {
        List<KqLockAttenda> list = new ArrayList<>();
        if ("".equals(LOCK_TABLE)) {
            return list;
        }
        RecordSet rs = new RecordSet();
        StringBuilder sql = new StringBuilder();
        sql.append("select * from " + LOCK_TABLE + " where ");
        sql.append(" " + LOCK_LOCK + "=1 ");
        kqLog.info("sql>>>>>>>>>>>>>>>" + sql.toString());

        KqLockAttenda bean = null;
        rs.execute(sql.toString());
        while (rs.next()) {
            bean = new KqLockAttenda();
            String iscompany = Util.null2String(rs.getString(LOCK_COM));
            bean.setLOCK_COM(iscompany);
            String subcompanyIds = Util.null2String(rs.getString(LOCK_SUBCOM));
            bean.setLOCK_SUBCOM(subcompanyIds);
            String departmentIds = Util.null2String(rs.getString(LOCK_DEPT));
            bean.setLOCK_DEPT(departmentIds);
            String resouceIds = Util.null2String(rs.getString(LOCK_RESOURCE));
            bean.setLOCK_RESOURCE(resouceIds);
            String fromdate = Util.null2String(rs.getString(LOCK_FROMDATE));
            bean.setLOCK_FROMDATE(fromdate);
            String todate = Util.null2String(rs.getString(LOCK_TODATE));
            bean.setLOCK_TODATE(todate);
            list.add(bean);
        }

        return list;
    }

    public boolean checkLockStatus(String resourceId, String kqdate) {
        if (!LOCK_DEGREE.equals("0")&&!LOCK_DEGREE.equals("1")) {
            return false;
        }
        try {
            RecordSet rs = new RecordSet();
            ResourceComInfo resourceComInfo = new ResourceComInfo();
            String subcompanyId = resourceComInfo.getSubCompanyID(resourceId);
            String departmentId = resourceComInfo.getDepartmentID(resourceId);
            String sql = "select * from " + LOCK_TABLE + " where (" + LOCK_FROMDATE + "<='" + kqdate + "' and  " + LOCK_TODATE + " >= '" + kqdate + "' ) ";
            String sqlWhere = "";
            if (!"".equals(resourceId)) {
                if (rs.getDBType().equals("oracle")||rs.getDBType().equals("postgresql")) {
                    sqlWhere=" and (','||CAST(" + LOCK_RESOURCE + " AS VARCHAR(2000))||',') like '%," + resourceId + ",%'";
                } else if (rs.getDBType().equals("mysql")) {
                    sqlWhere=" and concat(','," + LOCK_RESOURCE + ",',') like '%," + resourceId + ",%'" ;
                } else {
                    sqlWhere=" and (','+CAST(" + LOCK_RESOURCE + " AS VARCHAR(2000))+',') like '%," + resourceId + ",%'" ;
                }
                rs.executeQuery(sql + sqlWhere);
                rs.writeLog("KQLockAttendaUtil>sql="+sql + sqlWhere);
                if (rs.next()) {
                    String lockValue = rs.getString(LOCK_LOCK);
                    if(lockValue.equals("1")){
                        return true;
                    }else{
                        return false;
                    }
                }
            }
            if (!"".equals(departmentId)) {
                if (rs.getDBType().equals("oracle")||rs.getDBType().equals("postgresql")) {
                    sqlWhere=" and (','||CAST(" + LOCK_DEPT + " AS VARCHAR(2000))||',') like '%," + departmentId + ",%'";
                } else if (rs.getDBType().equals("mysql")) {
                    sqlWhere=" and  concat(','," + LOCK_DEPT + ",',') like '%," + departmentId + ",%'";
                } else {
                    sqlWhere=" and  (','+CAST(" + LOCK_DEPT + " AS VARCHAR(2000))+',') like '%," + departmentId + ",%'";
                }
                rs.writeLog("KQLockAttendaUtil>sql="+sql + sqlWhere);
                rs.executeQuery(sql + sqlWhere);
                if (rs.next()) {
                    String lockValue = rs.getString(LOCK_LOCK);
                    if(lockValue.equals("1")){
                        return true;
                    }else{
                        return false;
                    }
                }
            }
            if (!"".equals(subcompanyId)) {
                if (rs.getDBType().equals("oracle")||rs.getDBType().equals("postgresql")) {
                    sqlWhere=" and  (','||CAST(" + LOCK_SUBCOM + " AS VARCHAR(2000))||',') like '%," + subcompanyId + ",%'";
                } else if (rs.getDBType().equals("mysql")) {
                    sqlWhere=" and  concat(','," + LOCK_SUBCOM + ",',') like '%," + subcompanyId + ",%'";
                } else {
                    sqlWhere=" and  (','+CAST(" + LOCK_SUBCOM + " AS VARCHAR(2000))+',') like '%," + subcompanyId + ",%'";
                }
                rs.writeLog("KQLockAttendaUtil>sql="+sql + sqlWhere);
                rs.executeQuery(sql + sqlWhere);
                if (rs.next()) {
                    String lockValue = rs.getString(LOCK_LOCK);
                    if(lockValue.equals("1")){
                        return true;
                    }else{
                        return false;
                    }
                }
            }
            sqlWhere=" and (" + LOCK_COM + "=1) ";
            rs.writeLog("KQLockAttendaUtil>sql="+sql + sqlWhere);
            rs.executeQuery(sql + sqlWhere);
            if (rs.next()) {
                String lockValue = rs.getString(LOCK_LOCK);
                if(lockValue.equals("1")){
                    return true;
                }else{
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }
    /**
     * 判断当前范围内是否存在已被锁定的日期
     *
     * @param fromDate 开始日期
     * @param toDate   结束日期
     */
    public boolean isLockDate(String fromDate, String toDate) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        try {
            Date startDate = format.parse(fromDate);
            Date endDate = format.parse(toDate);
            for (KqLockAttenda datas : this.sdkqbb_map) {
                Date ksrqDate = format.parse(datas.getLOCK_FROMDATE());
                Date jsrqDate = format.parse(datas.getLOCK_TODATE());
                if (isInterSection(startDate, endDate, ksrqDate, jsrqDate)) {
                    return true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }
    /**
     * 两个时间段是否存在交叉
     *
     * @param startDateOne 第一个时间段的开始时间
     * @param endDateOne   第一个时间段的结束时间
     * @param startDateTwo 第二个时间段的开始时间
     * @param endDateTwo   第二个时间段的结束时间
     */
    private static boolean isInterSection(Date startDateOne, Date endDateOne, Date startDateTwo, Date endDateTwo) {
        Date maxStartDate = startDateOne;
        if (maxStartDate.before(startDateTwo)) {
            maxStartDate = startDateTwo;
        }
        Date minEndDate = endDateOne;
        if (endDateTwo.before(minEndDate)) {
            minEndDate = endDateTwo;
        }
        return maxStartDate.before(minEndDate) || (maxStartDate.getTime() == minEndDate.getTime());
    }



    public boolean isLockDate(Map<String, Object> params) {
        JSONObject jsonObj = JSON.parseObject(Util.null2String(params.get("data")));
        String fromDate = Util.null2String(jsonObj.getString("fromDate"));
        String toDate = Util.null2String(jsonObj.getString("toDate"));
        String typeselect = Util.null2String(jsonObj.getString("typeselect"));
        if (typeselect.length() == 0) {
            typeselect = "3";
        }
        if (!typeselect.equals("0") && !typeselect.equals("6")) {
            if (typeselect.equals("1")) {
                fromDate = TimeUtil.getCurrentDateString();
                toDate = TimeUtil.getCurrentDateString();
            } else {
                fromDate = TimeUtil.getDateByOption(typeselect, "0");
                toDate = TimeUtil.getDateByOption(typeselect, "1");
            }
        }
        return this.isLockDate(fromDate, toDate);
    }

    /**
     * 锁定日期范围内的考勤报表数据
     *
     * @param params 参数
     * @param user   当前登录用户
     */
    public Map<String, Object> lockReport(Map<String, Object> params, User user) {
        Map<String, Object> result = new HashMap<>();
        BaseBean baseBean = new BaseBean();
        JSONObject jsonObj = JSON.parseObject(Util.null2String(params.get("data")));
        String fromDate = Util.null2String(jsonObj.getString("fromDate"));
        String toDate = Util.null2String(jsonObj.getString("toDate"));
        String typeselect = Util.null2String(jsonObj.getString("typeselect"));
        if (typeselect.length() == 0) {
            typeselect = "3";
        }
        if (!typeselect.equals("0") && !typeselect.equals("6")) {
            if (typeselect.equals("1")) {
                fromDate = TimeUtil.getCurrentDateString();
                toDate = TimeUtil.getCurrentDateString();
            } else {
                fromDate = TimeUtil.getDateByOption(typeselect, "0");
                toDate = TimeUtil.getDateByOption(typeselect, "1");
            }
        }
        boolean lockDate = this.isLockDate(fromDate, toDate);
        if (lockDate) {
            result.put("status", "0");
            result.put("message", "当前日期范围内存在被锁定的数据，请重新选择日期范围!");
            return result;
        }
        // 保存锁定周期
        RecordSet recordSet = new RecordSet();
        String formModeId = baseBean.getPropValue("qc2648525", "sdkqbb.form.mode.id");
        String sql = "insert into uf_sdkqbb (ksrq, jsrq, sfsd, formmodeid) values ('" + fromDate + "', '" + toDate + "', 1, " + formModeId + ")";
        recordSet.execute(sql);
//        RunnableTask runnableTask = new RunnableTask(user, fromDate, toDate, params, null, true, true);
//        EXECUTOR_SERVICE.execute(runnableTask);
//        String finalFromDate = fromDate;
//        String finalToDate = toDate;
//        new Thread(() -> {
//            // 更新初始化缓存
//            this.sdkqbb_map = this.initModuleData();
//            // 未归档流程退回到发起节点
//            this.doForceOver(user);
//            // 同步数据到建模表
//            this.syncKqData(user, finalFromDate, finalToDate, params);
//        }).start();
        result.put("status", "1");
        result.put("message", "锁定成功!");
        return result;
    }

    /**
     * 强制归档
     */
    public void doForceOver(User user) {
        BaseBean baseBean = new BaseBean();
        String workflowIds = baseBean.getPropValue("qc2648525", "kq.workflow.ids");
        RecordSet recordSet = new RecordSet();
        RecordSet newRecordSet = new RecordSet();
        String newSql = "select requestid, workflowid, requestnamenew from workflow_requestbase where workflowid in (" + workflowIds + ") and (currentnodetype = 1 or currentnodetype = 2)";
        String updateSql = "update workflow_requestbase set requestnamenew = ? where requestid = ?";
        recordSet.execute(newSql);
        while (recordSet.next()) {
            String requestId = Util.null2String(recordSet.getString("requestid"));
            String workflowId = Util.null2String(recordSet.getString("workflowid"));
            String requestNameNew = Util.null2String(recordSet.getString("requestnamenew"));
            if (this.canContinueWorkflow(workflowId, requestId)) {
                continue;
            }
            Map<String, Object> otherParams = new HashMap<>();
            otherParams.put("ismonitor", "1");
            ReqOperateRequestEntity reqOperateRequestEntity = new ReqOperateRequestEntity();
            reqOperateRequestEntity.setRequestId(Util.getIntValue(requestId));
            reqOperateRequestEntity.setRemark("流程日期存在被锁定的考勤数据，流程强制归档且不计入考勤数据");
            reqOperateRequestEntity.setOtherParams(otherParams);
            this.doForceOver(user, reqOperateRequestEntity);
            requestNameNew = "(已作废)" + requestNameNew;
            newRecordSet.executeUpdate(updateSql, requestNameNew, requestId);
        }
    }

    /**
     * 考勤数据同步到建模表
     *
     * @param fromDate 开始日期
     * @param toDate   结束日期
     */
    public void syncKqData(User user, String fromDate, String toDate, Map<String, Object> params) {
        try {
            BigDecimal hourMins = new BigDecimal("60");
            ResourceComInfo resourceComInfo = new ResourceComInfo();
            RecordSet recordSet = new RecordSet();
            BatchRecordSet batchRecordSet = new BatchRecordSet();
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            BaseBean baseBean = new BaseBean();
            String tableName = baseBean.getPropValue("qc2648525", "sync.table.name");
            String formModeId = baseBean.getPropValue("qc2648525", "sync.form.mode.id");
            KQReportBiz kqReportBiz = new KQReportBiz();
            List<Map<String, Object>> allLeaveRules = KQLeaveRulesBiz.getAllLeaveRules();
            Map<String, Object> flowData = kqReportBiz.getDailyFlowData(params, user);
            Date startDate = simpleDateFormat.parse(fromDate);
            LocalDate l1 = LocalDate.parse(fromDate);
            LocalDate l2 = LocalDate.parse(toDate);
            long fromEpochDay = l1.toEpochDay();
            long toEpochDay = l2.toEpochDay();
            long diff = toEpochDay - fromEpochDay;
            for (long i = 0; i <= diff; i++) {
                List<List<Object>> values = new ArrayList<>();
                String syncDate = simpleDateFormat.format(DateUtil.addDay(startDate, (int) i));
                String newSql = "select * from kq_format_total where kqdate = ?";
                recordSet.executeQuery(newSql, syncDate);
                while (recordSet.next()) {
                    List<Object> list = new ArrayList<>();
                    list.add(formModeId);
                    String xm = recordSet.getString("resourceid");
                    String gs = resourceComInfo.getSubCompanyID(xm);
                    String bm = resourceComInfo.getDepartmentID(xm);
                    String rq = recordSet.getString("kqdate");
                    String bc = Util.null2s(recordSet.getString("serialid"), "-1");
                    list.add(xm);
                    list.add(gs);
                    list.add(bm);
                    list.add(rq);
                    list.add(bc);

                    Map<String, Object> signDetailInfo = getSignDetailInfo(xm, rq, user);
                    String sbdk1 = Util.null2s(Util.null2String(signDetailInfo.get("signintime1")), "-1");
                    String qdkqjg1 = Util.null2s(Util.null2String(signDetailInfo.get("signinstatus1")), "-1");
                    String xbdk1 = Util.null2s(Util.null2String(signDetailInfo.get("signouttime1")), "-1");
                    String qtkqjg1 = Util.null2s(Util.null2String(signDetailInfo.get("signoutstatus1")), "-1");
                    list.add(sbdk1);
                    list.add(qdkqjg1);
                    list.add(xbdk1);
                    list.add(qtkqjg1);
                    String sbdk2 = Util.null2s(Util.null2String(signDetailInfo.get("signintime2")), "-1");
                    String qdkqjg2 = Util.null2s(Util.null2String(signDetailInfo.get("signinstatus2")), "-1");
                    String xbdk2 = Util.null2s(Util.null2String(signDetailInfo.get("signouttime2")), "-1");
                    String qtkqjg2 = Util.null2s(Util.null2String(signDetailInfo.get("signoutstatus2")), "-1");
                    list.add(sbdk2);
                    list.add(qdkqjg2);
                    list.add(xbdk2);
                    list.add(qtkqjg2);
                    String sbdk3 = Util.null2s(Util.null2String(signDetailInfo.get("signintime3")), "-1");
                    String qdkqjg3 = Util.null2s(Util.null2String(signDetailInfo.get("signinstatus3")), "-1");
                    String xbdk3 = Util.null2s(Util.null2String(signDetailInfo.get("signouttime3")), "-1");
                    String qtkqjg3 = Util.null2s(Util.null2String(signDetailInfo.get("signoutstatus3")), "-1");
                    list.add(sbdk3);
                    list.add(qdkqjg3);
                    list.add(xbdk3);
                    list.add(qtkqjg3);

                    double ycqts = Util.getDoubleValue(recordSet.getString("workdays"), 0);
                    BigDecimal ygzsc = new BigDecimal(Util.null2s(recordSet.getString("workmins"), "0")).divide(hourMins, 2, BigDecimal.ROUND_HALF_DOWN);
                    double sjcqts = Util.getDoubleValue(recordSet.getString("attendancedays"), 0);
                    BigDecimal sjgzsc = new BigDecimal(Util.null2s(recordSet.getString("attendanceMins"), "0")).divide(hourMins, 2, BigDecimal.ROUND_HALF_DOWN);
                    list.add(ycqts);
                    list.add(ygzsc.doubleValue());
                    list.add(sjcqts);
                    list.add(sjgzsc.doubleValue());

                    double cd = Util.getDoubleValue(recordSet.getString("beLate"), 0);
                    BigDecimal cdsc = new BigDecimal(Util.null2s(recordSet.getString("beLateMins"), "0")).divide(hourMins, 2, BigDecimal.ROUND_HALF_DOWN);
                    double zt = Util.getDoubleValue(recordSet.getString("leaveEearly"), 0);
                    BigDecimal ztsc = new BigDecimal(Util.null2s(recordSet.getString("leaveEarlyMins"), "0")).divide(hourMins, 2, BigDecimal.ROUND_HALF_DOWN);
                    double kg = Util.getDoubleValue(recordSet.getString("absenteeism"), 0);
                    BigDecimal kgsc = new BigDecimal(Util.null2s(recordSet.getString("absenteeismMins"), "0")).divide(hourMins, 2, BigDecimal.ROUND_HALF_DOWN);
                    list.add(cd);
                    list.add(cdsc.doubleValue());
                    list.add(zt);
                    list.add(ztsc.doubleValue());
                    list.add(kg);
                    list.add(kgsc.doubleValue());

                    Map<String, String> leaveInfo = getLeaveInfo(syncDate, xm, allLeaveRules, flowData);
                    double leaveType_2 = Util.getDoubleValue(leaveInfo.get("leaveType_2"), 0);
                    double leaveType_5 = Util.getDoubleValue(leaveInfo.get("leaveType_5"), 0);
                    double leaveType_6 = Util.getDoubleValue(leaveInfo.get("leaveType_6"), 0);
                    double leaveType_7 = Util.getDoubleValue(leaveInfo.get("leaveType_7"), 0);
                    double leaveType_8 = Util.getDoubleValue(leaveInfo.get("leaveType_8"), 0);
                    double leaveType_9 = Util.getDoubleValue(leaveInfo.get("leaveType_9"), 0);
                    double leaveType_10 = Util.getDoubleValue(leaveInfo.get("leaveType_10"), 0);
                    double leaveType_11 = Util.getDoubleValue(leaveInfo.get("leaveType_11"), 0);
                    double leaveType_12 = Util.getDoubleValue(leaveInfo.get("leaveType_12"), 0);
                    double leaveType_13 = Util.getDoubleValue(leaveInfo.get("leaveType_13"), 0);
                    double leaveType_14 = Util.getDoubleValue(leaveInfo.get("leaveType_14"), 0);
                    double leaveType_15 = Util.getDoubleValue(leaveInfo.get("leaveType_15"), 0);
                    double leaveType_16 = Util.getDoubleValue(leaveInfo.get("leaveType_16"), 0);
                    double leaveType_17 = Util.getDoubleValue(leaveInfo.get("leaveType_17"), 0);
                    list.add(leaveType_2);
                    list.add(leaveType_5);
                    list.add(leaveType_6);
                    list.add(leaveType_7);
                    list.add(leaveType_8);
                    list.add(leaveType_9);
                    list.add(leaveType_10);
                    list.add(leaveType_11);
                    list.add(leaveType_12);
                    list.add(leaveType_13);
                    list.add(leaveType_14);
                    list.add(leaveType_15);
                    list.add(leaveType_16);
                    list.add(leaveType_17);

                    double bgldxgzrjb = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|workingDayOvertime_nonleave")));
                    bgldxgzrjb = bgldxgzrjb < 0 ? 0 : bgldxgzrjb;
                    double bgldxzmjb = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|restDayOvertime_nonleave")));
                    bgldxzmjb = bgldxzmjb < 0 ? 0 : bgldxzmjb;
                    double bgldxjjqjb = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|holidayOvertime_nonleave")));
                    bgldxjjqjb = bgldxjjqjb < 0 ? 0 : bgldxjjqjb;
                    double gldxgzrjb = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|workingDayOvertime_4leave")));
                    gldxgzrjb = gldxgzrjb < 0 ? 0 : gldxgzrjb;
                    double gldxzmjb = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|restDayOvertime_4leave")));
                    gldxzmjb = gldxzmjb < 0 ? 0 : gldxzmjb;
                    double gldxjjrjb = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|holidayOvertime_4leave")));
                    gldxjjrjb = gldxjjrjb < 0 ? 0 : gldxjjrjb;
                    double jbzj = Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(String.valueOf(bgldxgzrjb + bgldxzmjb + bgldxjjqjb + gldxgzrjb + gldxzmjb + gldxjjrjb)));
                    list.add(bgldxgzrjb);
                    list.add(bgldxzmjb);
                    list.add(bgldxjjqjb);
                    list.add(gldxgzrjb);
                    list.add(gldxzmjb);
                    list.add(gldxjjrjb);
                    list.add(jbzj);

                    double cc = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|businessLeave")));
                    cc = cc < 0 ? 0 : cc;
                    double gc = Util.getDoubleValue(Util.null2String(flowData.get(xm + "|" + syncDate + "|officialBusiness")));
                    gc = gc < 0 ? 0 : gc;
                    double lq1 = Util.getDoubleValue(recordSet.getString("forgotcheck"), 0);
                    double lq2 = Util.getDoubleValue(recordSet.getString("forgotBeginWorkCheck"), 0);
                    double lq = lq1 + lq2;
                    list.add(cc);
                    list.add(gc);
                    list.add(lq);

                    values.add(list);
                }
                // 删除之前同步的数据
                newSql = "delete from " + tableName + " where rq = ?";
                recordSet.executeUpdate(newSql, syncDate);
                // 同步新数据到到建模表
                newSql = "insert into " + tableName + "(formmodeid,xm,gs,bm,rq,bc,sbdk1,qdkqjg1,xbdk1,qtkqjg1,sbdk2,qdkqjg2,xbdk2,qtkqjg2,sbdk3,qdkqjg3,xbdk3,qtkqjg3,ycqts,ygzsc,sjcqts,sjgzsc,cd,cdsc,zt,ztsc,kg,kgsc,leaveType_2,leaveType_5,leaveType_6,leaveType_7,leaveType_8,leaveType_9,leaveType_10,leaveType_11,leaveType_12,leaveType_13,leaveType_14,leaveType_15,leaveType_16,leaveType_17,bgldxgzrjb,bgldxzmjb,bgldxjjqjb,gldxgzrjb,gldxzmjb,gldxjjrjb,jbzj,cc,gc,lq) " +
                        " values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
                batchRecordSet.executeBatchSqlNew(newSql, values);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static Map<String, String> getLeaveInfo(String kqdate, String id, List<Map<String, Object>> allLeaveRules, Map<String, Object> flowData) {
        Map<String, String> map = new HashMap<>();
        for (int i = 0; allLeaveRules != null && i < allLeaveRules.size(); i++) {
            Map<String, Object> leaveRule = allLeaveRules.get(i);
            String flowType = Util.null2String("leaveType_" + leaveRule.get("id"));
            String leaveData = Util.null2String(flowData.get(id + "|" + kqdate + "|" + flowType));
            String flowLeaveBackType = Util.null2String("leavebackType_" + leaveRule.get("id"));
            String leavebackData = Util.null2s(Util.null2String(flowData.get(id + "|" + kqdate + "|" + flowLeaveBackType)), "0.0");
            String b_flowLeaveData = "";
            String flowLeaveData = "";
            try {
                if (leaveData.length() == 0) {
                    leaveData = "0.0";
                }
                if (leavebackData.length() == 0) {
                    leavebackData = "0.0";
                }
                BigDecimal b_leaveData = new BigDecimal(leaveData);
                BigDecimal b_leavebackData = new BigDecimal(leavebackData);
                b_flowLeaveData = b_leaveData.subtract(b_leavebackData).toString();
                if (Util.getDoubleValue(b_flowLeaveData, -1) < 0) {
                    b_flowLeaveData = "0.0";
                }
            } catch (Exception e) {
                kqLog.info("GetKQReportCmd:leaveData" + leaveData + ":leavebackData:" + leavebackData + ":" + e);
            }
            if (b_flowLeaveData.length() > 0) {
                flowLeaveData = KQDurationCalculatorUtil.getDurationRound(b_flowLeaveData);
            } else {
                flowLeaveData = KQDurationCalculatorUtil.getDurationRound(Util.null2String(Util.getDoubleValue(leaveData, 0.0) - Util.getDoubleValue(leavebackData, 0.0)));
            }
            map.put(flowType, flowLeaveData);
        }
        return map;
    }

    public static Map<String, Object> getSignDetailInfo(String resourceId, String kqDate, User user) {
        Map<String, Object> data = new HashMap<>();
        Map<String, Object> signStatusInfo = null;
        RecordSet rs = new RecordSet();
        KQTimesArrayComInfo kqTimesArrayComInfo = new KQTimesArrayComInfo();
        try {
            String sql = " select kqdate,resourceid,serialid,serialnumber,workbegindate,workbegintime, " +
                    " workenddate,workendtime,workmins,signindate,signintime,signoutdate,signouttime, \n" +
                    " attendanceMins,belatemins,graveBeLateMins,leaveearlymins,graveLeaveEarlyMins,absenteeismmins,forgotcheckMins,forgotBeginWorkCheckMins," +
                    " leaveMins,leaveInfo,evectionMins,outMins,signinid,signoutid \n" +
                    " from kq_format_detail b \n" +
                    " where resourceid = " + resourceId + " and kqdate ='" + kqDate + "' \n" +
                    " order by serialnumber \n";
            rs.execute(sql);
            while (rs.next()) {
                String serialid = Util.null2String(rs.getString("serialid"));
                int serialnumber = rs.getInt("serialnumber") + 1;
                String workbegindate = Util.null2String(rs.getString("workbegindate")).trim();
                String workbegintime = Util.null2String(rs.getString("workbegintime")).trim();
                String workenddate = Util.null2String(rs.getString("workenddate")).trim();
                String workendtime = Util.null2String(rs.getString("workendtime")).trim();
                String signintime = Util.null2String(rs.getString("signintime")).trim();
                String signouttime = Util.null2String(rs.getString("signouttime")).trim();
                String beLateMins = Util.null2String(rs.getString("beLateMins")).trim();
                String graveBeLateMins = Util.null2String(rs.getString("graveBeLateMins")).trim();
                String leaveEarlyMins = Util.null2String(rs.getString("leaveEarlyMins")).trim();
                String graveLeaveEarlyMins = Util.null2String(rs.getString("graveLeaveEarlyMins")).trim();
                String absenteeismMins = Util.null2String(rs.getString("absenteeismMins")).trim();
                String forgotCheckMins = Util.null2String(rs.getString("forgotcheckMins")).trim();
                String forgotBeginWorkCheckMins = Util.null2String(rs.getString("forgotBeginWorkCheckMins")).trim();
                String signinid = Util.null2String(rs.getString("signinid")).trim();
                String signoutid = Util.null2String(rs.getString("signoutid")).trim();
                int leaveMins = rs.getInt("leaveMins");
                String leaveInfo = Util.null2String(rs.getString("leaveInfo"));
                int evectionMins = rs.getInt("evectionMins");
                int outMins = rs.getInt("outMins");

                if (serialid.length() > 0) {
                    if (workbegintime.length() > 0) {
                        signStatusInfo = new HashMap<>();
                        signStatusInfo.put("workdate", workbegindate);
                        signStatusInfo.put("worktime", workbegintime);
                        signStatusInfo.put("beLateMins", beLateMins);
                        signStatusInfo.put("forgotBeginWorkCheckMins", forgotBeginWorkCheckMins);
                        signStatusInfo.put("graveBeLateMins", graveBeLateMins);
                        signStatusInfo.put("absenteeismMins", absenteeismMins);
                        signStatusInfo.put("leaveMins", leaveMins);
                        signStatusInfo.put("leaveInfo", leaveInfo);
                        signStatusInfo.put("evectionMins", evectionMins);
                        signStatusInfo.put("outMins", outMins);
                        data.put("signintime" + serialnumber, signintime.length() == 0 ? SystemEnv.getHtmlLabelName(25994, user.getLanguage()) : signintime);
                        data.put("signinstatus" + serialnumber, KQReportBiz.getSignStatus(signStatusInfo, user, "on"));
                    }

                    if (workendtime.length() > 0) {
                        signStatusInfo = new HashMap<>();
                        signStatusInfo.put("workdate", workenddate);
                        signStatusInfo.put("worktime", kqTimesArrayComInfo.turn48to24Time(workendtime));
                        signStatusInfo.put("leaveEarlyMins", leaveEarlyMins);
                        signStatusInfo.put("graveLeaveEarlyMins", graveLeaveEarlyMins);
                        signStatusInfo.put("forgotCheckMins", forgotCheckMins);
                        signStatusInfo.put("forgotBeginWorkCheckMins", forgotBeginWorkCheckMins);
                        signStatusInfo.put("absenteeismMins", absenteeismMins);
                        signStatusInfo.put("leaveMins", leaveMins);
                        signStatusInfo.put("leaveInfo", leaveInfo);
                        signStatusInfo.put("evectionMins", evectionMins);
                        signStatusInfo.put("outMins", outMins);
                        data.put("signouttime" + serialnumber, signouttime.length() == 0 ? SystemEnv.getHtmlLabelName(25994, user.getLanguage()) : signouttime);
                        data.put("signoutstatus" + serialnumber, KQReportBiz.getSignStatus(signStatusInfo, user, "off"));
                    }
                } else {
                    signStatusInfo = new HashMap();
                    signStatusInfo.put("leaveMins", leaveMins);
                    signStatusInfo.put("leaveInfo", leaveInfo);
                    signStatusInfo.put("evectionMins", evectionMins);
                    signStatusInfo.put("outMins", outMins);
                    if (signinid.length() > 0) {
                        data.put("signintime" + serialnumber, signintime.length() == 0 ? SystemEnv.getHtmlLabelName(25994, user.getLanguage()) : signintime);
                        data.put("signinstatus" + serialnumber, KQReportBiz.getSignStatus(signStatusInfo, user, "on"));
                        if (signoutid.length() > 0) {
                            data.put("signouttime" + serialnumber, signouttime.length() == 0 ? SystemEnv.getHtmlLabelName(25994, user.getLanguage()) : signouttime);
                            data.put("signoutstatus" + serialnumber, KQReportBiz.getSignStatus(signStatusInfo, user, "off"));
                        }
                    } else {
                        data.put("signinstatus" + serialnumber, KQReportBiz.getSignStatus(signStatusInfo, user, "on"));
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return data;
    }


    /**
     * 考勤流程是否能继续提交
     *
     * @param request 请求详情
     */
    public String canSubmitWorkflow(RequestInfo request) {
        String requestId = request.getRequestid();
        String workflowId = request.getWorkflowid();
        kqLog.info("do canSubmitWorkflow on workflowId:" + workflowId + "|requestId:" + requestId);
        if (canContinueWorkflow(workflowId, requestId)) {
            return Action.SUCCESS;
        }
        request.getRequestManager().setMessageid("11111" + request.getRequestid() + "22222");
        request.getRequestManager().setMessagecontent("当前日期周期内存在被锁定的考勤数据，请重新选择日期周期!");
        return Action.FAILURE_AND_CONTINUE;
    }

    /**
     * 考勤流程能否继续提交
     *
     * @param workflowId 流程id
     * @param requestId  请求id
     */
    public boolean canContinueWorkflow(String workflowId, String requestId) {
        try {
            int requestIdInt = Util.getIntValue(requestId, 0);
            String formId = new WorkflowComInfo().getFormId(workflowId);
            // 是否是考勤变更流程
            boolean isProcessChange = false;
            // 是否是销假流程
            boolean isLeaveBack = false;
            List<SplitBean> splitBeans = new ArrayList<>();
            RecordSet recordSet = new RecordSet();
            String sql = "select * from kq_att_proc_set where field001 = ? and field002 = ? ";
            recordSet.executeQuery(sql, workflowId, formId);
            if (recordSet.next()) {
                String proc_set_id = recordSet.getString("id");
                String useDetails = recordSet.getString("usedetail");
                int kqType = Util.getIntValue(recordSet.getString("field006"));
                Map<String, String> map = new HashMap<>();
                if (requestIdInt > 0) {
                    map.put("requestId", "and t.requestId = " + requestIdInt);
                }
                KQFlowActiontBiz kqFlowActiontBiz = new KQFlowActiontBiz();
                ResourceComInfo resourceComInfo = new ResourceComInfo();
                DateTimeFormatter datetimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
                Map<String, String> sqlMap = kqFlowActiontBiz.handleSql(proc_set_id, useDetails, requestIdInt, kqType, map);
                if (kqType == KqSplitFlowTypeEnum.LEAVE.getFlowtype()) {
                    KQFlowLeaveUtil kqFlowLeaveUtil = new KQFlowLeaveUtil();
                    kqFlowLeaveUtil.handleKQLeaveAction(sqlMap, splitBeans, datetimeFormatter, Util.getIntValue(workflowId), requestIdInt, resourceComInfo);
                } else if (kqType == KqSplitFlowTypeEnum.EVECTION.getFlowtype()) {
                    KQFlowEvectionUtil kqFlowEvectionUtil = new KQFlowEvectionUtil();
                    kqFlowEvectionUtil.handleKQEvectionAction(sqlMap, splitBeans, datetimeFormatter, Util.getIntValue(workflowId), requestIdInt, resourceComInfo);
                } else if (kqType == KqSplitFlowTypeEnum.OUT.getFlowtype()) {
                    KQFlowOutUtil kqFlowOutUtil = new KQFlowOutUtil();
                    kqFlowOutUtil.handleKQOutAction(sqlMap, splitBeans, datetimeFormatter, Util.getIntValue(workflowId), requestIdInt, resourceComInfo);
                } else if (kqType == KqSplitFlowTypeEnum.OVERTIME.getFlowtype()) {
                    KQFlowOvertimeUtil kqFlowOvertimeUtil = new KQFlowOvertimeUtil();
                    kqFlowOvertimeUtil.handleKQOvertimeAction(sqlMap, splitBeans, datetimeFormatter, Util.getIntValue(workflowId), requestIdInt, resourceComInfo, "");
                } else if (kqType == KqSplitFlowTypeEnum.CARD.getFlowtype()) {
                    String[] cardFields = KQAttFlowFieldsSetBiz.cardFields;
                    RecordSet newRecordSet = new RecordSet();
                    if (!sqlMap.isEmpty()) {
                        for (Map.Entry<String, String> entry : sqlMap.entrySet()) {
                            String value = entry.getValue();
                            newRecordSet.execute(value);
                            while (newRecordSet.next()) {
                                String fDetailSignDate = cardFields[3];
                                String signDate = Util.null2s(newRecordSet.getString(fDetailSignDate), "");
                                if (this.isLockDate(signDate, signDate)) {
                                    return false;
                                }
                            }
                        }
                    }
                } else if (kqType == KqSplitFlowTypeEnum.LEAVEBACK.getFlowtype()) {
                    isLeaveBack = true;
                    KQFlowLeaveBackUtil kqFlowLeaveBackUtil = new KQFlowLeaveBackUtil();
                    kqFlowLeaveBackUtil.handleKQLeaveBackAction(sqlMap, splitBeans, datetimeFormatter, Util.getIntValue(workflowId), requestIdInt, resourceComInfo);
                } else if (kqType == KqSplitFlowTypeEnum.PROCESSCHANGE.getFlowtype()) {
                    isProcessChange = true;
                    KQFlowProcessChangeUtil kqFlowProcessChangeUtil = new KQFlowProcessChangeUtil();
                    kqFlowProcessChangeUtil.handleKQProcessChangeAction(sqlMap, splitBeans, datetimeFormatter, Util.getIntValue(workflowId), requestIdInt, resourceComInfo);
                }
            }
            for (SplitBean splitBean : splitBeans) {
                String fromDate = "";
                String toDate = "";
                if (isProcessChange) {
                    ProcessChangeSplitBean processChangeSplitBean = splitBean.getProcessChangeSplitBean();
                    fromDate = processChangeSplitBean.getFromdatedb();
                    toDate = processChangeSplitBean.getTodatedb();
                    if (this.isLockDate(fromDate, toDate)) {
                        return false;
                    }
                    fromDate = splitBean.getFromdatedb();
                    toDate = splitBean.getTodatedb();
                } else if (isLeaveBack) {
                    String resourceId = splitBean.getResourceId();
                    String leaveBackRequestId = splitBean.getLeavebackrequestid();
                    sql = "select fromdatedb, todatedb from kq_flow_split_leave where resourceid = ? and requestid = ? group by fromdatedb, todatedb";
                    recordSet.executeQuery(sql, resourceId, leaveBackRequestId);
                    if (recordSet.next()) {
                        fromDate = Util.null2String(recordSet.getString("fromdatedb"));
                        toDate = Util.null2String(recordSet.getString("todatedb"));
                    }
                } else {
                    fromDate = splitBean.getFromdatedb();
                    toDate = splitBean.getTodatedb();
                }
                if (this.isLockDate(fromDate, toDate)) {
                    return false;
                }
            }
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * 强制归档
     *
     * @param user         用户
     * @param requestParam 请求参数
     */
    public void doForceOver(User user, ReqOperateRequestEntity requestParam) {
        int requestId = requestParam.getRequestId();
        Map<String, Object> otherParams = requestParam.getOtherParams();
        PAResponseEntity pAEntity = RequestOperateBiz.verifyBefore(user, requestId, otherParams);
        if (pAEntity.getCode() != null) {
            return;
        }
        WfFunctionManageUtil wfFunctionManageUtil = new WfFunctionManageUtil();
        RequestInfoEntity reqEntity = RequestOperateBiz.initRequestInfo(requestParam, user);
        int isremark = reqEntity.getIsremark();
        int agentType = Util.getIntValue(Util.null2String(otherParams.get("agenttype")));
        int agentorByAgentId = Util.getIntValue(Util.null2String(otherParams.get("agentorbyagentid")));
        // 流程代理出去，本人提交，需先收回代理
        if (isremark == 2 && agentType == 1 && agentorByAgentId > 0) {
            AgentManager agentManager = new AgentManager(user);
            agentManager.agentBackRequest(agentorByAgentId, user.getUID(), reqEntity.getWorkflowId() + "", requestParam.getRequestId());
            reqEntity = RequestOperateBiz.initRequestInfo(requestParam, user);
            isremark = reqEntity.getIsremark();
        }
        int nodeid = Util.getIntValue(reqEntity.getCurrentNodeId());
        int takisremark = reqEntity.getTakisremark();
        HashMap map = wfFunctionManageUtil.wfFunctionManageByNodeid(reqEntity.getWorkflowId(), FreeNodeBiz.isFreeNode(nodeid) ? -9 : nodeid);
        WfForceOver wfo = new WfForceOver();
        String ov = (String) map.get("ov");
        boolean hasRight = (isremark == 0 && (takisremark == 0 || takisremark == -1)) && "1".equals(ov) && wfo.isNodeOperator(requestId, user.getUID()) && !reqEntity.getCurrentNodeType().equals("3");
        boolean isMonitor = "1".equals(Util.null2String(otherParams.get("ismonitor")));
        if (!hasRight && isMonitor) {
            MonitorDTO monitorInfo = new Monitor().getMonitorInfo(String.valueOf(user.getUID()), reqEntity.getCreatorId(), String.valueOf(reqEntity.getWorkflowId()));
            hasRight = monitorInfo.getIsforceover() && !"3".equals(reqEntity.getCurrentNodeType());
        }
        if (hasRight) {
            ArrayList requestidlist = new ArrayList();
            requestidlist.add("" + requestId);
            wfo.setRemark(requestParam.getRemark());
            wfo.doForceOver(requestidlist, null, null, user);
            pAEntity.setCode(PAResponseCode.SUCCESS);
            String isnew = new BaseBean().getPropValue("WorkflowOvertimeIsNew", "isNew");
            if ("1".equals(isnew)) {
                OvertimeBiz.getInstance().cancelOvertimeTask(requestId, Util.getIntValue(reqEntity.getCurrentNodeId()));// 取消超时任务
            }
        } else {
            pAEntity.setCode(PAResponseCode.NO_PERMISSION);
        }
    }

    /**
     * 同步建模表里面的打卡数据
     */
    public void importData() {
        RecordSet rs = new RecordSet();
        String endDateTime = weaver.common.DateUtil.getDate(new Date(), "yyyy-MM-dd HH:mm");
        Calendar calendar = weaver.common.DateUtil.getCalendar(endDateTime);
        calendar.add(Calendar.MINUTE, -45);
        String startDateTime = weaver.common.DateUtil.getDate(calendar.getTime(), "yyyy-MM-dd HH:mm");

        KQScheduleSignImport kqScheduleSignImport = new KQScheduleSignImport();
        BatchRecordSet bRs = new BatchRecordSet();
        List<List<Object>> lsParams = new ArrayList<>();
        List<Object> params = null;
        List<List<Object>> lsDelParams = new ArrayList<>();
        List<Object> delParams = null;
        List<String> lsFormatData = new ArrayList<>();
        List<List<Object>> lsFormatParams = new ArrayList<>();
        List<Object> formatParams = null;
        Map<String, Map<String, String>> users = kqScheduleSignImport.getUsers(7);
        Map<String, String> loginidMap = users.get("loginidMap");
        Map<String, String> lastnameMap = users.get("lastnameMap");
        Map<String, String> workcodeMap = users.get("workcodeMap");
        List<String> keymap = new ArrayList<>();
        List<ImportSetting> lsImportSetting = new ArrayList<>();
        ImportSetting importSetting = null;
        String sql = " select datasourceid, loginid, workcode, lastname, signdate, signtime, tablename, clientaddress,longitude,latitude,addr,memo from HrmScheduleSignSet ";
        rs.execute(sql);
        while (rs.next()) {
            importSetting = new ImportSetting();
            importSetting.setDatasourceid(Util.null2String(rs.getString("datasourceid")).trim());
            importSetting.setLoginid(Util.null2String(rs.getString("loginid")).trim());
            importSetting.setWorkcode(Util.null2String(rs.getString("workcode")).trim());
            importSetting.setLastname(Util.null2String(rs.getString("lastname")).trim());
            importSetting.setSigndate(Util.null2String(rs.getString("signdate")).trim());
            importSetting.setSigntime(Util.null2String(rs.getString("signtime")).trim());
            importSetting.setTablename(Util.null2String(rs.getString("tablename")).trim());
            importSetting.setClientaddress(Util.null2String(rs.getString("clientaddress")).trim());
            importSetting.setLongitude(Util.null2String(rs.getString("longitude")).trim());
            importSetting.setLatitude(Util.null2String(rs.getString("latitude")).trim());
            importSetting.setAddr(Util.null2String(rs.getString("addr")).trim());
            importSetting.setMemo(Util.null2String(rs.getString("memo")).trim());
            lsImportSetting.add(importSetting);
        }
        List<KQScheduleSignEntity> lsKQScheduleSignData = new ArrayList<>();
        KQScheduleSignEntity kqScheduleSignData = null;
        for (int i = 0; i < lsImportSetting.size(); i++) {
            try {
                importSetting = lsImportSetting.get(i);
                String datasourceid = importSetting.getDatasourceid();
                String loginid = importSetting.getLoginid();
                String workcode = importSetting.getWorkcode();
                String lastname = importSetting.getLastname();
                String signdate = importSetting.getSigndate();
                String signtime = importSetting.getSigntime();
                String tablename = importSetting.getTablename();
                String clientaddress = importSetting.getClientaddress();
                String longitude = importSetting.getLongitude();
                String latitude = importSetting.getLatitude();
                String addr = importSetting.getAddr();
                String memo = importSetting.getMemo();
                DateFormat df = new SimpleDateFormat("HH:mm:ss");
                if ("sqlserver".equals(rs.getDBType())) {
                    sql = "select * from " + tablename + " where " + " " + signdate + "+' '+" + signtime + " >= '" + startDateTime + "' and " + signdate + "+' '+" + signtime + " <= '" + endDateTime + "' ";
                } else {
                    sql = "select * from " + tablename + " where " + " concat(" + signdate + ",' '," + signtime + ") >= '" + startDateTime + "' and concat(" + signdate + ",' '," + signtime + ") <= '" + endDateTime + "' ";
                }
                // 有遇到外部同步，上面的【conn = ds.getConnection();】这个阻塞住，导致下面的同步不执行，下面的用法和集成领导确认
                RecordSetDataSource rsds = new RecordSetDataSource(datasourceid);
                rsds.execute(sql);
                while (rsds.next()) {
                    String tmpLoginid = "";
                    String tmpWorkcode = "";
                    String tmpLastname = "";
                    String tmpSigndate = "";
                    String tmpSigntime = "";
                    String tmpClientaddress = "";
                    String tmpLongitude = "";
                    String tmpLatitude = "";
                    String tmpAddr = "";
                    String tmpMemo = "";
                    if (loginid.length() > 0) {
                        tmpLoginid = Util.null2String(rsds.getString(loginid)).trim();
                    }
                    if (workcode.length() > 0) {
                        tmpWorkcode = Util.null2String(rsds.getString(workcode)).trim();
                    }
                    if (lastname.length() > 0) {
                        tmpLastname = Util.null2String(rsds.getString(lastname)).trim();
                    }
                    if (signdate.length() > 0) {
                        tmpSigndate = Util.null2String(rsds.getString(signdate)).trim();
                    }
                    if (clientaddress.length() > 0) {
                        tmpClientaddress = Util.null2String(rsds.getString(clientaddress));
                    }
                    if (longitude.length() > 0) {
                        tmpLongitude = Util.null2String(rsds.getString(longitude)).trim();
                    }
                    if (latitude.length() > 0) {
                        tmpLatitude = Util.null2String(rsds.getString(latitude)).trim();
                    }
                    if (addr.length() > 0) {
                        tmpAddr = Util.null2String(rsds.getString(addr)).trim();
                    }
                    if (memo.length() > 0) {
                        tmpMemo = Util.null2String(rsds.getString(memo)).trim();
                    }
                    if (signtime.length() > 0) {
                        tmpSigntime = Util.null2String(rsds.getString(signtime)).trim();
                        // 判断tmpSigntime格式 08:21:11 如果为长格式需要格式化
                        if (tmpSigntime.length() > 8) {
                            // 有遇到signtime格式为hh:mm:ss.0000的格式导致同步不成功
                            if (tmpSigntime.length() < 16) {
                                tmpSigntime = tmpSigntime.substring(0, 8);
                            }
                            // 比如上yyyy-mm-dd hh:mm的格式走原逻辑
                            else {
                                tmpSigntime = df.format(Timestamp.valueOf(tmpSigntime));
                            }
                        } else if (tmpSigntime.length() < 8) {
                            // 不带秒的情况 自动补齐
                            tmpSigntime += ":00";
                        }
                    } else {
                        tmpSigntime = "";
                    }

                    if (tmpSigndate.length() > 10) {
                        // 如果时间为长格式，从signdate字段中格式化时间
                        if (tmpSigndate.length() == 16) {
                            //不带秒的情况  自动补齐
                            tmpSigntime += ":00";
                        }
                        // 以signtime字段为有限
                        if (tmpSigntime.length() == 0) {
                            tmpSigndate = tmpSigndate.length() > 19 ? tmpSigndate.substring(0, 19) : tmpSigndate;
                            tmpSigntime = df.format(Timestamp.valueOf(tmpSigndate));
                        }
                        tmpSigndate = Tools.formatDate(tmpSigndate, "yyyy-MM-dd");
                    }
                    // 如果时间格式不带秒补齐
                    if (tmpSigntime.length() <= 5) {
                        tmpSigntime += ":00";
                    }
                    kqScheduleSignData = new KQScheduleSignEntity();
                    kqScheduleSignData.setLoginid(tmpLoginid);
                    kqScheduleSignData.setWorkcode(tmpWorkcode);
                    kqScheduleSignData.setLastname(tmpLastname);
                    kqScheduleSignData.setSigndate(tmpSigndate);
                    kqScheduleSignData.setSigntime(tmpSigntime);
                    kqScheduleSignData.setClientaddress(tmpClientaddress);
                    kqScheduleSignData.setLongitude(tmpLongitude);
                    kqScheduleSignData.setLatitude(tmpLatitude);
                    kqScheduleSignData.setAddr(tmpAddr);
                    kqScheduleSignData.setMemo(tmpMemo);
                    lsKQScheduleSignData.add(kqScheduleSignData);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        List<String> noRepertSynData = new ArrayList<>();
        for (int i = 0; i < lsKQScheduleSignData.size(); i++) {
            kqScheduleSignData = lsKQScheduleSignData.get(i);
            String tmpLoginid = kqScheduleSignData.getLoginid();
            String tmpWorkcode = kqScheduleSignData.getWorkcode();
            String tmpLastname = kqScheduleSignData.getLastname();
            String tmpSigndate = kqScheduleSignData.getSigndate();
            String tmpSigntime = kqScheduleSignData.getSigntime();
            String tmpClientaddress = kqScheduleSignData.getClientaddress();
            String tmpLongitude = kqScheduleSignData.getLongitude();
            String tmpLatitude = kqScheduleSignData.getLatitude();
            String tmpAddr = kqScheduleSignData.getAddr();
            String tmpMemo = kqScheduleSignData.getMemo();
            int userid = 0;
            if (Util.null2String(tmpLoginid).length() > 0) {
                userid = weaver.common.StringUtil.parseToInt(loginidMap.get(tmpLoginid), 0);
            }
            if (userid == 0 && Util.null2String(tmpWorkcode).length() > 0) {
                userid = weaver.common.StringUtil.parseToInt(workcodeMap.get(tmpWorkcode), 0);
            }
            if (userid == 0 && Util.null2String(tmpLastname).length() > 0) {
                userid = weaver.common.StringUtil.parseToInt(lastnameMap.get(tmpLastname), 0);
            }
            if (userid <= 0) {
                continue;
            }
            String dkParamData = userid + "#" + tmpSigndate + "#" + tmpSigntime;
            if (noRepertSynData.contains(dkParamData)) {
                continue;
            }
            noRepertSynData.add(dkParamData);
            params = new ArrayList<>();
            params.add(userid);
            params.add(1);
            params.add(tmpSigndate);
            params.add(tmpSigntime);
            params.add(tmpClientaddress);
            params.add(1);
            params.add(1);
            params.add("OutDataSourceSyn");
            params.add(tmpLongitude);
            params.add(tmpLatitude);
            params.add(tmpAddr);
            params.add(tmpMemo);
            lsParams.add(params);
            String belongDate = getBelongDate(new User(userid), tmpSigndate, tmpSigntime);
            String formatData = userid + "|" + tmpSigndate + "|" + belongDate;
            if (!lsFormatData.contains(formatData)) {
                lsFormatData.add(formatData);
            }
        }
        Map<String, List<String>> overtimeMap = Maps.newHashMap();
        List<String> overtimeList = Lists.newArrayList();
        // 刷新报表数据
        for (int i = 0; i < lsFormatData.size(); i++) {
            formatParams = new ArrayList<>();
            String key = lsFormatData.get(i);
            String[] formatData = Util.splitString(lsFormatData.get(i), "|");
            String date_1 = weaver.common.DateUtil.addDate(formatData[1], -1);
            formatParams.add(formatData[0]);
            formatParams.add(date_1);
            lsFormatParams.add(formatParams);
            formatParams = new ArrayList<>();
            formatParams.add(formatData[0]);
            formatParams.add(formatData[1]);
            lsFormatParams.add(formatParams);
            if (!keymap.contains(key)) {
                keymap.add(key);
                delParams = new ArrayList<>();
                delParams.add(formatData[0]);
                // delParams.add(formatData[1]);
                delParams.add(startDateTime + ":00");
                delParams.add(endDateTime + ":00");
                lsDelParams.add(delParams);
            }
            String resourceId = formatData[0];
            String kqdate = formatData[1];
            String belongDate = formatData[2];
            if (overtimeMap.containsKey(resourceId)) {
                List<String> tmp_overtimeList = overtimeMap.get(resourceId);
                if (!tmp_overtimeList.contains(kqdate)) {
                    tmp_overtimeList.add(kqdate);
                    if (!kqdate.equals(belongDate)) {
                        tmp_overtimeList.add(belongDate);
                    }
                }
            } else {
                if (!overtimeList.contains(kqdate)) {
                    overtimeList.add(kqdate);
                    if (!kqdate.equals(belongDate)) {
                        overtimeList.add(belongDate);
                    }
                }
                overtimeMap.put(resourceId, overtimeList);
            }
        }
        // 删除本次同步数据
        // sql = " delete from hrmschedulesign where signfrom='OutDataSourceSyn' and userid =? and signdate = ? ";
        if ("sqlserver".equals(rs.getDBType())) {
            sql = "delete from hrmschedulesign where signfrom = 'OutDataSourceSyn' and userid = ? and signdate + ' ' + signtime >= ? and signdate + ' ' + signtime <= ?";
        } else {
            sql = "delete from hrmschedulesign where signfrom = 'OutDataSourceSyn' and userid = ? and concat(signdate,' ',signtime) >= ? and concat(signdate,' ',signtime) <= ?";
        }
        bRs.executeBatchSql(sql, lsDelParams);
        sql = " insert into HrmScheduleSign (userid, usertype, signdate, signtime, clientaddress, isincom, isimport, signfrom, longitude, latitude, addr,memo) values(?,?,?,?,?,?,?,?,?,?,?,?)";
        bRs.executeBatchSql(sql, lsParams);
        new KQFormatBiz().format(lsFormatParams);
        // 处理加班生成
        List<KQTaskBean> tasks = new ArrayList<>();
        for (Map.Entry<String, List<String>> mme : overtimeMap.entrySet()) {
            String resid = mme.getKey();
            List<String> overList = mme.getValue();
            for (String date : overList) {
                SplitActionUtil.pushOverTimeTasks(date, date, resid, tasks);
            }
        }
        if (!tasks.isEmpty()) {
            KQQueue.writeTasks(tasks);
        }
    }

    public String getBelongDate(User user, String curDate, String signDateTime) {
        String belongDate = curDate;
        Map<String, Object> todayLineMap = KQDurationCalculatorUtil.getWorkButton(user, curDate, true);
        if (todayLineMap != null) {
            List<Map<String, String>> preInfo = (ArrayList<Map<String, String>>) todayLineMap.get("pre_signTime");
            if (preInfo != null) {
                for (Map<String, String> entry : preInfo) {
                    String pre_endtime_across = entry.get("endtime_across");
                    String pre_endtime = entry.get("endtime");
                    String signEndInfo = "1".equals(pre_endtime_across) ? (curDate + " " + pre_endtime + ":00") : (weaver.common.DateUtil.addDate(curDate, -1) + " " + pre_endtime + ":00");
                    String signOutDateTime = signDateTime;
                    if (signOutDateTime.compareTo(signEndInfo) < 0) {
                        belongDate = weaver.common.DateUtil.addDate(curDate, -1);
                    }
                }
            }
        }
        return belongDate;
    }

    /**
     * 线程池执行任务封装
     */
//    public static class RunnableTask implements Runnable {
//
//        private final User user;
//
//        private final boolean isSyncData;
//
//        private final boolean isDoForceOver;
//
//        private final String finalFromDate;
//
//        private final String finalToDate;
//
//        private Map<String, Object> params;
//
//        private final List<List<Object>> formatValues;
//
//        public RunnableTask(User user, String finalFromDate, String finalToDate, Map<String, Object> params, List<List<Object>> formatValues, boolean isSyncData, boolean isDoForceOver) {
//            this.user = user;
//            this.finalFromDate = finalFromDate;
//            this.finalToDate = finalToDate;
//            this.params = params;
//            this.formatValues = formatValues;
//            this.isSyncData = isSyncData;
//            this.isDoForceOver = isDoForceOver;
//        }
//
//        @Override
//        public void run() {
//            BaseBean baseBean = new BaseBean();
//            QC2648525Util qc2648525Util = new QC2648525Util();
//            if (formatValues != null && formatValues.size() > 0) {
//                long startTime = System.currentTimeMillis();
//                for (List<Object> objs : formatValues) {
//                    String userId = Util.null2String(objs.get(0));
//                    String kqDate = Util.null2String(objs.get(1));
//                    new KQFormatData().formatKqDate(userId, kqDate);
//                }
//                baseBean.writeLog("QC2648525 刷新考勤报表耗时:" + ((System.currentTimeMillis() - startTime) / 1000) + "秒");
//            }
//            if (isDoForceOver) {
//                long startTime = System.currentTimeMillis();
//                qc2648525Util.doForceOver(user);
//                baseBean.writeLog("QC2648525 强制归档流程执行耗时:" + ((System.currentTimeMillis() - startTime) / 1000) + "秒");
//            }
//            if (isSyncData) {
//                long startTime = System.currentTimeMillis();
//                if (params == null || params.isEmpty()) {
//                    Map<Object, Object> datas = new HashMap<>();
//                    datas.put("typeselect", "6");
//                    datas.put("fromDate", finalFromDate);
//                    datas.put("toDate", finalToDate);
//                    datas.put("attendanceSerial", "");
//                    datas.put("isFromMyAttendance", "1");
//                    params = new HashMap<>();
//                    params.put("data", JSONObject.toJSONString(datas));
//                    params.put("reportType", "month");
//                }
//                qc2648525Util.syncKqData(user, finalFromDate, finalToDate, params);
//                baseBean.writeLog("QC2648525 建模数据同步执行耗时:" + ((System.currentTimeMillis() - startTime) / 1000) + "秒");
//            }
//        }
//    }

    // --- 王宇负责部分 ---

    public static String[] colors = new String[]{"#47A9FC", "#70C06F", "#F6D65C", "#EA744D", "#4EBCCC", "#EA5C85", "#8F72F2", "#7297F2", "#CA4DE4", "#A7CC62"};

    public static String getKQGroupId(String userId, String workDate, boolean containFixed) {
        String groupId = "";
        KQGroupComInfo kqGroupComInfo = new KQGroupComInfo();
        KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
        List<String> lsGroupIds = kqGroupMemberComInfo.getKQGroups(userId,workDate);
        for (int i = 0; lsGroupIds != null && i < lsGroupIds.size(); i++) {
            String tmpGroupId = Util.null2String((String) lsGroupIds.get(i));
            String kqtype = kqGroupComInfo.getKqtype(tmpGroupId);
            String validity = kqGroupComInfo.getValidity(tmpGroupId);
            String validityfromdate = kqGroupComInfo.getValidityfromdate(tmpGroupId);
            String validityenddate = kqGroupComInfo.getValidityenddate(tmpGroupId);
            if (validity.equals("1")) {
                if (weaver.common.DateUtil.isInDateRange(workDate, validityfromdate, validityenddate)) {
                    groupId = tmpGroupId;
                } else {
                    continue;
                }
            } else {
                groupId = tmpGroupId;
            }
            if (groupId.length() > 0) {
                if (containFixed && kqtype.equals("2")) {
                    KQShiftScheduleComInfo kqShiftScheduleComInfo = new KQShiftScheduleComInfo();
                    if (Util.null2String(kqShiftScheduleComInfo.getSerialId(userId, workDate)).length() > 0 && Util.null2String(kqShiftScheduleComInfo.getGroupId(userId, workDate)).equals(groupId)) {
                        break;
                    }
                } else {
                    break;
                }
            }
        }
        return groupId;
    }

    /**
     * 从考勤报表中获取班次
     *
     * @param resourceId 用户
     * @param kqDate     日期
     * @param groupId    原来的groupId
     */
    public static String getGroupId(String resourceId, String kqDate, String groupId) {

        String newGroupId = "";
        String currentDate = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
        if (kqDate.compareTo(currentDate) > 0) {
            return groupId;
        }
        RecordSet recordSet = new RecordSet();
        String sql = "select groupid from kq_format_total where resourceid = ? and kqdate = ?";
        recordSet.executeQuery(sql, resourceId, kqDate);
        if (recordSet.next()) {
            newGroupId = Util.null2String(recordSet.getString("groupid"));
        }
        return StringUtil.isBlank(newGroupId) ? groupId : newGroupId;
    }

    /**
     * 前台我的排班颜色添加
     *
     * @param user       登录用户
     * @param resourceId 展示的用户id
     * @param fromDate   开始日期
     * @param toDate     结束日期
     * @param result     结果集
     */
    public static void getScheduleReportInfo(User user, String resourceId, String fromDate, String toDate, Map<String, Object> result) {
        RecordSet recordSet = new RecordSet();
        KQGroupComInfo kqGroupComInfo = new KQGroupComInfo();
        ShiftManagementToolKit shiftManagementToolKit = new ShiftManagementToolKit();
        KQWorkTime kqWorkTime = new KQWorkTime();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String sql = "";
        Map<String, Object> data = null;
        String currentDate = weaver.common.DateUtil.getCurrentDate();
        // 今天之前的日期在考勤报表中查询
        if (fromDate.compareTo(currentDate) <= 0) {
            // 结束日期小于当前日期 当前日期=结束日期
            if (toDate.compareTo(currentDate) < 0) {
                LocalDate localDate = LocalDate.parse(toDate);
                currentDate = formatter.format(localDate);
            }
            sql = "select kqdate, groupid, serialid from kq_format_total where resourceid = ? and kqdate >= ? and kqdate <= ?";
            recordSet.executeQuery(sql, resourceId, fromDate, currentDate);
            while (recordSet.next()) {
                String date = Util.null2String(recordSet.getString("kqdate"));
                String serialId = Util.null2String(recordSet.getString("serialid"));
                String groupId = Util.null2String(recordSet.getString("groupid"));
                if (StringUtil.isBlank(serialId)) {
                    continue;
                }
                String color = "";
                String kqType = kqGroupComInfo.getKqtype(groupId);
                if ("1".equals(kqType)) {
                    color = getColorBySerialId(kqGroupComInfo, groupId, serialId);
                } else if ("2".equals(kqType)) {
                    color = getColorBySerialId(kqGroupComInfo, groupId, serialId);
                }
                String serial = shiftManagementToolKit.getShiftOnOffWorkSections(serialId, user.getLanguage());
                data = new HashMap<>();
                data.put("serial", serial);
                data.put("bgColor", color);
                result.put(date, data);
            }
        }
        // 今天之后的日期 固定班次保持不变，排班在排班表中查询
        currentDate = weaver.common.DateUtil.getCurrentDate();

        if (toDate.compareTo(currentDate) >= 0) {

            // 开始日期大于当前日期 当前日期=开始日期
            LocalDate localDate;
            if (fromDate.compareTo(currentDate) > 0) {
                localDate = LocalDate.parse(fromDate);
            } else {
                localDate = LocalDate.parse(currentDate).plusDays(1);
            }
            currentDate = formatter.format(localDate);
            String kqGroupId = getKQGroupId(resourceId, currentDate, true);
            LocalDate startDate = LocalDate.parse(currentDate);
            LocalDate endDate = LocalDate.parse(toDate);
            KQHolidaySetComInfo kqHolidaySetComInfo = new KQHolidaySetComInfo();
            long sd = startDate.toEpochDay();
            long ed = endDate.toEpochDay();
            long diff = ed - sd;
            for (long i = 0; i <= diff; i++) {
                LocalDate td = startDate.plusDays(i);
                String targetDate = formatter.format(td);
                String changeType = kqHolidaySetComInfo.getChangeType(kqGroupId, targetDate);
                if ("1".equals(changeType) || "3".equals(changeType)) {
                    continue;
                }
                WorkTimeEntity workTime = kqWorkTime.getWorkTime(resourceId, targetDate);
                String serialId = workTime.getSerialId();
                if (serialId != null) {
                    data = new HashMap<>();
                    String serial = shiftManagementToolKit.getShiftOnOffWorkSections(serialId, user.getLanguage());
                    data.put("serial", serial);
                    data.put("bgColor", "#000");
                    result.put(targetDate, data);
                }
            }
        }
    }

    public static String getColorBySerialId(KQGroupComInfo kqGroupComInfo, String groupId, String serialId) {
        String color = "";
        String serialIds = kqGroupComInfo.getSerialids(groupId);
        if (StringUtil.isNotBlank(serialIds)) {
            String[] serialArr = serialIds.split(",");
            for (int i = 0; i < serialArr.length; i++) {
                if (serialArr[i].trim().equals(serialId)) {
                    color = colors[i];
                    break;
                }
            }
        }
        return color;
    }

    /**
     * 星期
     *
     * @param date 日期
     */
    public static int getDayOfWeek(String date) {
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            // 创建Calendar类实例
            Calendar instance = Calendar.getInstance();
            // 根据指定日期获取周几
            instance.setTime(format.parse(date));
            // 因为数组下标从0开始，而返回的是数组的内容，是数组{1,2,3,4,5,6,7}中用1~7来表示 Saturday->7
            int i = instance.get(Calendar.DAY_OF_WEEK);
            if (i >= Calendar.MONDAY && i <= Calendar.FRIDAY) {
                i = i - 2;
            } else if (i == Calendar.SATURDAY) {
                i = 5;
            } else if (i == Calendar.SUNDAY) {
                i = 6;
            }
            return i;
        } catch (Exception e) {
            return -1;
        }
    }

}
